#!/usr/bin/env python
import sys
import rospy
import math
import time
import numpy as np
from dji_sdk.srv import *
from dji_sdk.msg import * # equivalent to MissionWpUploadRequest (i think)

def activate_sdk():
    rospy.wait_for_service('dji_sdk/activation')
    try:
        activation_proxy = rospy.ServiceProxy('dji_sdk/activation', Activation)
        activation_request = ActivationRequest() # should be empty anyway
        activation_response = activation_proxy(activation_request)
        print "Result of activation request: %s"%str(activation_response.result)
        return activation_response
    except rospy.ServiceException, e:
        print "Service call failed: %s"%e

def obtain_control_authority():
    rospy.wait_for_service('dji_sdk/sdk_control_authority')
    try:
        control_authority_proxy = rospy.ServiceProxy('dji_sdk/sdk_control_authority', SDKControlAuthority)
        authority_request = SDKControlAuthorityRequest() # there is only a single field here 
        authority_request.control_enable = 1 # take control
        authority_response = control_authority_proxy(authority_request)
        print "Result of control authority request: %s"%str(authority_response.result)
        return authority_response
    except rospy.ServiceException, e:
        print "Service call failed: %s"%e

def drone_task(task_id = 6):
    rospy.wait_for_service('dji_sdk/drone_task_control')
    try:
        task_proxy = rospy.ServiceProxy('dji_sdk/drone_task_control', DroneTaskControl)
        task_request = DroneTaskControlRequest() # there is only a single field here 
        task_request.task = task_id # takeoff action (4 for takeoff, 6 for landing)
        task_response = task_proxy(task_request)
        print "Result of task request: %s"%str(task_response.result)
        return task_response
    except rospy.ServiceException, e:
        print "Service call failed: %s"%e

def get_mission_info():
    rospy.wait_for_service('dji_sdk/mission_waypoint_getInfo')
    try:
        info_proxy = rospy.ServiceProxy('dji_sdk/mission_waypoint_getInfo', MissionWpGetInfo)
        info_request = MissionWpGetInfoRequest() # empty anyway
        info_response = info_proxy(info_request)
        print "Information retrieved"
        return info_response
    except rospy.ServiceException, e:
        print "Service call failed: %s"%e

def begin_mission(mission_action = 0):
    rospy.wait_for_service('dji_sdk/mission_waypoint_action')
    try:
        action_proxy = rospy.ServiceProxy('dji_sdk/mission_waypoint_action', MissionWpAction)
        action_request = MissionWpActionRequest()
        action_request.action = mission_action # 0 for start, 1 for stop
        action_response = action_proxy(action_request)
        print "Result of task request: %s"%action_response.result
        return action_response
    except rospy.ServiceException, e:
        print "Service call failed: %s"%e


def waypoint_upload_client(latitude_list, longitude_list):
    rospy.wait_for_service('dji_sdk/mission_waypoint_upload')
    print 'Latitude list: '
    for a in latitude_list: print(a)
    print 'longitude list: '
    for a in longitude_list: print(a)
    try:
        mission_waypoint_upload_proxy = rospy.ServiceProxy('dji_sdk/mission_waypoint_upload', MissionWpUpload)
        # build the actual request
        req1 = MissionWpUploadRequest()
        req1.waypoint_task.velocity_range = np.float32(10.0)
        req1.waypoint_task.idle_velocity = np.float32(5.0)
        req1.waypoint_task.action_on_finish = np.uint8(0)
        req1.waypoint_task.mission_exec_times = np.uint8(1)
        req1.waypoint_task.yaw_mode = np.uint8(0)
        req1.waypoint_task.trace_mode = np.uint8(0)
        req1.waypoint_task.action_on_rc_lost = np.uint8(1)
        req1.waypoint_task.gimbal_pitch_mode = np.uint8(0)
        # build individual waypoints
        #
        req1.waypoint_task.mission_waypoint = [None]*len(latitude_list)
        for i in range(len(latitude_list)):
            req1.waypoint_task.mission_waypoint[i] = MissionWaypoint()
            req1.waypoint_task.mission_waypoint[i].latitude = latitude_list[i]
            req1.waypoint_task.mission_waypoint[i].longitude = longitude_list[i]
            req1.waypoint_task.mission_waypoint[i].altitude = 10.0
            req1.waypoint_task.mission_waypoint[i].damping_distance = 0.0
            req1.waypoint_task.mission_waypoint[i].target_yaw = 0
            req1.waypoint_task.mission_waypoint[i].target_gimbal_pitch = 0
            req1.waypoint_task.mission_waypoint[i].turn_mode = 0
            req1.waypoint_task.mission_waypoint[i].has_action = 0
            #~ req1.waypoint_task.mission_waypoint[i] = MissionWaypoint()
            #~ req1.waypoint_task.mission_waypoint[i].latitude = np.float64(latitude_list[i])
            #~ req1.waypoint_task.mission_waypoint[i].longitude = np.float64(longitude_list[i])
            #~ req1.waypoint_task.mission_waypoint[i].altitude = np.float32(10.0)
            #~ req1.waypoint_task.mission_waypoint[i].damping_distance = np.float32(0.0)
            #~ req1.waypoint_task.mission_waypoint[i].target_yaw = np.int16(0)
            #~ req1.waypoint_task.mission_waypoint[i].target_gimbal_pitch = np.int16(0)
            #~ req1.waypoint_task.mission_waypoint[i].turn_mode = np.uint8(0)
            #~ req1.waypoint_task.mission_waypoint[i].has_action = np.uint8(0)
            #~ req1.waypoint_task.mission_waypoint[-1].action_time_limit = np.uint8(0)
            #~ req1.waypoint_task.mission_waypoint[-1].waypoint_action = MissionWaypointAction()
            #~ req1.waypoint_task.mission_waypoint[-1].waypoint_action.action_repeat = 0
            #~ req1.waypoint_task.mission_waypoint[-1].waypoint_action.command_list = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
            #~ req1.waypoint_task.mission_waypoint[-1].waypoint_action.command_parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
            # print "waypoint num %s :"%str(i)
            # print tmp_waypoint
            # print "---------------"
        print "Filled out class:"
        print req1
        resp1 = mission_waypoint_upload_proxy(req1)
        print "Result of waypoint task upload: %s"%str(resp1.result)
        return resp1
    except rospy.ServiceException, e:
        print "Service call failed: %s"%e
        
def useage():
    return "%s MissionWaypointTask"%sys.argv[0]    
        
if __name__ == "__main__":
    activation_response = activate_sdk()
    control_response = obtain_control_authority()
    #~ takeoff_response = drone_task(4)
    #~ if takeoff_response.result:
        #~ print "Drone has taken off!"
        #~ #time.sleep(5)
    #~ else:
        #~ print "Something went wrong, landing and exiting"
        #~ drone_task(6)
        #~ quit()
        
    start_lat = 22.542812
    start_lon = 113.958902
    delta_var = 0.0005
    wp_lats = [start_lat, start_lat + delta_var, start_lat + delta_var, start_lat]
    wp_lons = [start_lon, start_lon, start_lon + delta_var, start_lon + delta_var]
    
    wp_lats.extend(wp_lats)
    wp_lons.extend(wp_lons)
    
    waypoint_upload_response = waypoint_upload_client(wp_lats, wp_lons)
    time.sleep(1)
    mission_info_response = get_mission_info()
    print mission_info_response
    
    #~ begin_response = begin_mission(0)
    #~ if begin_response.result:
        #~ print "Mission starting!"			
    #~ else:
        #~ print "Something went wrong, exiting"
        #~ drone_task(6)
        #~ quit()
    # land the drone where she sits
    #landing_response = drone_task(6)
    #if landing_response.result:
    #    print "Drone has landed!"
    #else:
    #    print "Drone something went wrong, exiting"
    #    quit()
        
        
